/*
 Demo for driving PIC16Fxx as I2C-Slave according to Application Note AN734
 Description: This example is indended to demonstrate several capabilities of I2C bus
             - master/slave communication
             - timeout in case of communiation failure
             - capability of sending unsigned char variables to 32 different
                internal UC registers
              - variable data lenght
 Mikrocontroller:       PIC16F1847
 Clock:                 internal clock (8.0 MHz)
 Compiler:              mikroC V7.01
 Developer:   V.O.     Mitroi Nicolae Alexandru   05 mai 2016
 Last updated:         paulfjujo 18 mai 2017
=======================================
*/

#define Version "170605"
// rajout renvoi mesure de temperature SLAVE -> Master


//#define Version "170603"
// rajout renvoi de la conversion de la valeur brute EA1 lue par Pic Maitre
// en format flottant Volts

//#define Version "170602"

// rajout reception mesure ADC1 par PIC Maitre
// renvoi string de 16 cars

//#define Version "170518"
#define Directory "C:\\_MikroC\\_MesProjets_MikroC\r\n \\_16Fxx_MASTER_SLAVE_I2C_MikroE\\PIC16F1847_I2CSlave_Protocol"
#define Projet "16F1847_Slave_I2C_2017.mcppi"
#define Source "PIC16F1847_I2CSlave_Protocol"      // sans extension !
#define MCU "16F1847"
#define OSCILLATEUR_INTERNE
#define FOSC "Interne 8MHz"
#define POWER_SUPPLY "3.7V"    // (3,3 to 4,2V battery)
// internal Oscillateur 8MHz
// CONFIG1 : $8007 : 0x39E4
// CONFIG2 : $8008 : 0x1613

  //-- commandes terminal VT220
#define CLS 12     // effacement de page sur Terminal VBRAY
#define CR 13
#define VT 10
#define LF 10
#define TAB 9
#define BELL 7


 #define _I2C_MAX_REGISTERS_COUNT  32


sbit Led_R at PORTA.B4;
sbit Led_R_Direction at TRISA4_bit;

unsigned char Buff_LCD[17];

char TEXTE[80];
char * txt;
char CRam1[64];
char Signature[17];
unsigned int i,j,k,l,m;

#define MAXLEN1 80
unsigned char buffer1[MAXLEN1];
unsigned int Index1=0;
unsigned int UART1_DataReady=0;
unsigned int i1=0;
int CptErr;
unsigned char c1;
unsigned int EA1 ; // valeur ADC EA1 (en Points) du PIC Maitre
float Volts; // valeur calcule par l'esclave et renvoye au Maitre

//int   Envoi_tension = 0;

void strConstRamCpy(char *dest, const char *source);
void UART1_Write_CText(const char *txt1);
void CRLF1(void);


 #include <PIC16F_I2C_SlaveFunctions_170602.h>

/* Interrupt Service Routine:
  Call:
  SSP Interrupt Flag (SSPIF), after each I2C event (when the master address
  write on the I2C bus or if there is data on the I2C bus)
  Tasks:
  Depending on the status there are different actions that can be made : 
  see comments  in source
  After each action, the SSP Interrupt Flag SSPIF must be cleared again
  Warning:
  The status reading or writing is always from the perspective of the master !!
*/

 void Interrupt() iv 0x0004 ics ICS_AUTO
{
    ManageI2C1Bus();
}



void Init_Hardware()
{
  TRISB  = 0xFF;
  TRISA  = 0xFF;
  //TRISA4_bit=0;    // led sur RA4
  Led_R_Direction =0;
  Led_R=0;
  ANSELA = 0b00000010;  // RA1=Analog
  ANSELB = 0;           // PORTB en Digital
 // If Configuration Word 1 , MCLR is enabled Weak Pull-up is always enabled.
  WPUA5_bit=1;
  WPUB=0;  // no pullup
  //Global WPUEN bit of the OPTION_REG register must be cleared for individual pull-ups to be enabled
  TRISB.B1 = 1;  // SDA1 I2C
  TRISB.B4 = 1;  // SCL1 I2C
  TRISB.B5 = 0;  // TX UART
  TRISB.B2 = 1;  // RX UART
  TRISB.B6 = 1;  // Timer1 input

  CM1CON0=0;       // disable comparators 1
  CM1CON1=0;
  CM2CON0=0;       // disable comparators  2
  CM2CON1=0;
}

#include "built_in.h"
#include "../16F1847_RB2_RB5_UART.inc"
//void Special_Init_UART1_RB2_RB5(void);
//void RAZ_UART1(void) ;
//void strConstRamCpy(char *dest, const char *source);
//void UART1_Write_CText(const char *txt1)  ;
//void CRLF1(void)     ;


void main(void)
{
//   + MCU initialization (using function InitI2CSlave
//   + All the logic is performed in ISR routine!!
  // Internal FOSC is 500Khz by default !
  #ifdef OSCILLATEUR_INTERNE
  OSCCON =0;
  OSCCON.IRCF3=1;         //8 MHz
  OSCCON.IRCF2=1;
  OSCCON.IRCF1=1;
  OSCCON.IRCF0=0;
  OSCCON.SCS1=1;  // internal FOSC
  OSCCON.SPLLEN=0;
  // --- config 32 Mhz  -----
  //OSCCON.SCS1=0;  // internal FOSC
  //OSCCON.SPLLEN=1;
  #endif
  
   Init_Hardware();
   //I2C1_Init(100000);
   Init_I2C1_Slave();
   txt=&TEXTE[0];
   UART1_Init(19200);   // Initialisation de l'UART
    // par defaut TX sur RB2  et RX sur RB1
   Delay_ms(100);
   Special_Init_UART1_RB2_RB5();

   UART1_Write(CLS);
   Delay_ms(500);
   UART1_Write_CText("\r\n Presentation :\r\n");
   UART1_Write_CText(" Directory : "Directory"\r\n");
   UART1_Write_CText(" Projet : "Projet"\r\n");
   UART1_Write_CText(" PIC : "MCU"  FOSC:  "FOSC"\r\n");
   UART1_Write_CText(" Source MikroC 7.01 : "Source"_"Version".C\r\n");
   // #define _I2C_CHIP_ADDRESS         0x40 //Holds the address received from I2C
   UART1_Write_CText(" 16F1847 en mode ESCLAVE I2C1 at @ 0x40\r\n");
   CRLF1();
   Delay_ms(1000);         //1234567890123456
   strConstRamCpy(Signature,"I am 16F Slave. ");

   j=0;

    while (1)
    {
         if (ucI2CRegistersStatus[1] == _I2C_STATUS_END)
          {
          
          UART1_Write_CText("Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2 ");
          ucI2CRegisters[2] = ucI2CRegisters[0] + ucI2CRegisters[1];
          ByteToStr( ucI2CRegisters[2],CRam1);
          UART1_Write_Text(CRam1);
         
          UART1_Write_CText(" Ecr.indice J ->#3 ");
          ucI2CRegisters[3]=j & 0x00FF;
          ByteToStr(j,CRam1);
          UART1_Write_Text(CRam1);
          
          UART1_Write_CText(" Signature Slave->reg.#4  #20 ");
          strncpy(ucI2CRegisters+4,Signature,16);
          

          UART1_Write_CText(" Lect.EA1 Maitre ->reg.#21,#22) ");
          EA1=ucI2CRegisters[21]<<8;
          EA1=EA1+ ucI2CRegisters[22];
          // et affichage local
          WordToStr(EA1,CRam1);
          UART1_Write_Text(CRam1);  UART1_Write_CText(" Pts\r\n");
          Volts=(float)EA1  * 2.048/1024.0;
          ucI2CRegisters[23]=0 ;
          // renvoi flottant vers MAITRE
          ucI2CRegisters[24]=Highest(Volts);
          ucI2CRegisters[25]=Higher(Volts);
          ucI2CRegisters[26]=Hi(Volts);
          ucI2CRegisters[27]=Lo(Volts);
          Delay_ms(10);

          
        #ifdef Debug_Registres
          for (i=0;i<27;i++)
          {
            ByteToHex(ucI2CRegisters[i],CRam1);
            UART1_Write_Text( CRam1);UART1_Write(" ");
          }
          CRLF1();
         #endif
          ucI2CRegistersStatus[1] = _I2C_STATUS_IDLE;
          }
         UART1_Write('*')   ;
         j++;
         if(j>64)
         {
           CRLF1();
           j=0;
           }
         Delay_ms(1500);
    }
}
/*

 Slave tout seul ...

 Presentation :
 Directory : C:\_MikroC\_MesProjets_MikroC
 \_16Fxx_MASTER_SLAVE_I2C_MikroE\PIC16F1847_I2CSlave_Protocol
 Projet : 16F1847_Slave_I2C_2017.mcppi
 PIC : 16F1847  FOSC:  Interne 8MHz
 Source MikroC 7.01 : PIC16F1847_I2CSlave_Protocol_170518.C
 16F1847 en mode ESCLAVE I2C1

*****************************************************************
******************************************

*/

/*
 Presentation :
 Directory : C:\_MikroC\_MesProjets_MikroC
 \_16Fxx_MASTER_SLAVE_I2C_MikroE\PIC16F1847_I2CSlave_Protocol
 Projet : 16F1847_Slave_I2C_2017.mcppi
 PIC : 16F1847  FOSC:  Interne 8MHz
 Source MikroC 7.01 : PIC16F1847_I2CSlave_Protocol_170603.C
 16F1847 en mode ESCLAVE I2C1 at @ 0x40

Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   0 Ecr.indice J ->#3   0 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)     0 Pts
***Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   1 Ecr.indice J ->#3   3 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   691 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   3 Ecr.indice J ->#3   4 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   680 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   5 Ecr.indice J ->#3   5 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   694 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   7 Ecr.indice J ->#3   6 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   703 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2   9 Ecr.indice J ->#3   7 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   704 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  11 Ecr.indice J ->#3   8 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   701 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  13 Ecr.indice J ->#3   9 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   704 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  15 Ecr.indice J ->#3  10 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   702 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  17 Ecr.indice J ->#3  11 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   658 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  19 Ecr.indice J ->#3  12 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   619 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  21 Ecr.indice J ->#3  13 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   554 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  23 Ecr.indice J ->#3  14 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   510 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  25 Ecr.indice J ->#3  15 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   488 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  27 Ecr.indice J ->#3  16 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   491 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  29 Ecr.indice J ->#3  17 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   487 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  31 Ecr.indice J ->#3  18 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   485 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  33 Ecr.indice J ->#3  19 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   525 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  35 Ecr.indice J ->#3  20 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   528 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  37 Ecr.indice J ->#3  21 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   521 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  39 Ecr.indice J ->#3  22 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   510 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  41 Ecr.indice J ->#3  23 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   512 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  43 Ecr.indice J ->#3  24 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   509 Pts
*Lect.reg.MAITRE-> #0,#1 Ecr.somme->#2  45 Ecr.indice J ->#3  25 Signature Slave->reg.#4  #20  Lect.EA1 Maitre ->reg.#21,#22)   510 Pts
*/




/*
0 1 mikroCPIC1618.exe -MSF -DBG -pP16F1847 -RA -C -O11111004 -fo8 -N"
C:\_MikroC\_MesProjets_MikroC\_16Fxx_MASTER_SLAVE_I2C_MikroE\PIC16F1847_I2CSlave_Protocol\PIC16F1847_I2CSlave_Protocol.mcppi" -SP"
C:\_MikroC\mikroC PRO for PIC\Defs\" -SP"
C:\_MikroC\mikroC PRO for PIC\Uses\P16_Enh\" -SP"
C:\_MikroC\_MesProjets_MikroC\_16Fxx_MASTER_SLAVE_I2C_MikroE\PIC16F1847_I2CSlave_Protocol\" -SP"
C:\_MikroC\_MesProjets_MikroC\_16Fxx_MASTER_SLAVE_I2C_MikroE\" -IP"
C:\_MikroC\_MesProjets_MikroC\_16Fxx_MASTER_SLAVE_I2C_MikroE\PIC16F1847_I2CSlave_Protocol\" "PIC16F_I2C_SlaveFunctions_170518.c" "PIC16F1847_I2CSlave_Protocol_170518.c" "
__Lib_Math.mcl" "
__Lib_MathDouble.mcl" "
__Lib_System.mcl" "
__Lib_Delays.mcl" "
__Lib_CString.mcl" "
__Lib_Conversions.mcl" "
__Lib_EEPROM.mcl" "
__Lib_I2C_b14b25.mcl" "
__Lib_UART_b21.mcl"
0 1139 Available RAM: 1008 [bytes], Available ROM: 8192 [bytes]
0 122 Compilation Started P16F1847.c
1400 123 Compiled Successfully P16F1847.c
0 122 Compilation Started
__Lib_Delays.c
123 123 Compiled Successfully
__Lib_Delays.c
0 122 Compilation Started PIC16F_I2C_SlaveFunctions_170518.c
53 1504 Initialization of extern object '_I2C_WRITE' PIC16F_I2C_SlaveFunctions_170518.c
54 1504 Initialization of extern object '_I2C_READ' PIC16F_I2C_SlaveFunctions_170518.c
55 1504 Initialization of extern object '_I2C_STATUS_IDLE' PIC16F_I2C_SlaveFunctions_170518.c
56 1504 Initialization of extern object '_I2C_STATUS_STARTED' PIC16F_I2C_SlaveFunctions_170518.c
57 1504 Initialization of extern object '_I2C_STATUS_INPROGRESS' PIC16F_I2C_SlaveFunctions_170518.c
58 1504 Initialization of extern object '_I2C_STATUS_ERROR' PIC16F_I2C_SlaveFunctions_170518.c
59 1504 Initialization of extern object '_I2C_STATUS_END' PIC16F_I2C_SlaveFunctions_170518.c
60 1504 Initialization of extern object '_I2C_ADDRESS' PIC16F_I2C_SlaveFunctions_170518.c
61 1504 Initialization of extern object '_I2C_DATA' PIC16F_I2C_SlaveFunctions_170518.c
503 123 Compiled Successfully PIC16F_I2C_SlaveFunctions_170518.c
75 1164 Variable 'tempVariable' has been eliminated by optimizer PIC16F_I2C_SlaveFunctions_170518.c
136 1164 Variable 'receivedMessageCount' has been eliminated by optimizer PIC16F_I2C_SlaveFunctions_170518.c
0 122 Compilation Started PIC16F1847_I2CSlave_Protocol_170518.c
0 1004 interrupt handler (Interrupt at 0x0004) PIC16F1847_I2CSlave_Protocol_170518.c
145 1509 Generated baud rate is 19231 bps (error = 0.16 percent) PIC16F1847_I2CSlave_Protocol_170518.c
185 123 Compiled Successfully PIC16F1847_I2CSlave_Protocol_170518.c
0 127 All files Compiled in 93 ms
0 1144 Used RAM (bytes): 247 (25%)  Free RAM (bytes): 761 (75%) Used RAM (bytes): 247 (25%)  Free RAM (bytes): 761 (75%)
0 1144 Used ROM (program words): 1223 (15%)  Free ROM (program words): 6969 (85%) Used ROM (program words): 1223 (15%)  Free ROM (program words): 6969 (85%)
0 125 Project Linked Successfully PIC16F1847_I2CSlave_Protocol.mcppi
0 128 Linked in 32 ms
0 129 Project 'PIC16F1847_I2CSlave_Protocol.mcppi' completed: 282 ms
0 103 Finished successfully: 18 mai 2017, 17:13:10 PIC16F1847_I2CSlave_Protocol.mcppi


*/